#
# spec file for package scotch
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor serial%{nil}

%define base_pname scotch
%define so_ver 0
%global _lto_cflags %{_lto_cflags} -ffat-lto-objects

%if "%{flavor}" == ""
ExclusiveArch:  do_not_build
 %define package_name %pname
%else
 %if "%{flavor}" == "serial"
 %else
%global mpi_flavor %{flavor}
 %endif
%endif

%if "%{flavor}" == "openmpi5"
ExcludeArch:    %{ix86} %{arm}
%endif

%bcond_without mumps

%{?mpi_flavor:%{bcond_without mpi}}%{!?mpi_flavor:%{bcond_with mpi}}

%if %{without mpi}
%define pname %{base_pname}
%define metis metis
%else
%define pt_pref pt
%define pname pt%{base_pname}
%define metis parmetis
%endif

%if %{without mpi}
 %define my_prefix %_prefix
 %define my_bindir %_bindir
 %define my_libdir %_libdir
 %define my_incdir %_includedir
 %define my_mandir %_mandir
%else
 %define my_suffix -%{mpi_flavor}
 %define my_prefix %{_libdir}/mpi/gcc/%{mpi_flavor}
 %define my_bindir %{my_prefix}/bin
 %define my_libdir %{my_prefix}/%{_lib}/
 %define my_incdir %{my_prefix}/include/
 %define my_mandir %{my_prefix}/share/man
%endif
%if 0%{!?package_name:1}
 %define package_name   %pname%{?my_suffix}
%endif
%define libname lib%{pname}%{so_ver}%{?my_suffix}
%define metis_pname  lib%{pname}-metis%{?my_suffix}

Summary:        Graph, mesh and hypergraph partitioning library
License:        CECILL-C
Group:          Productivity/Scientific/Math
Name:           %{package_name}
Version:        6.1.0
Release:        4.1
URL:            https://gitlab.inria.fr/scotch/scotch
Source0:        https://gitlab.inria.fr/scotch/scotch/-/archive/v%{version}/%{base_pname}-v%{version}.tar.gz
Source1:        scotch-Makefile.inc.in
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  bison
BuildRequires:  fdupes
BuildRequires:  flex
BuildRequires:  gcc
BuildRequires:  gcc-c++
BuildRequires:  gcc-fortran
BuildRequires:  libbz2-devel
BuildRequires:  make
BuildRequires:  zlib-devel
 %if %{with mpi}
BuildRequires:  %{mpi_flavor}-devel
 %endif

%description
Scotch is a software package for graph and mesh/hypergraph partitioning and
sparse matrix ordering.

%package     -n %{libname}
Summary:        Graph, mesh and hypergraph partitioning library
Group:          System/Libraries
 %if %{with mpi}
BuildRequires:  libscotch%{so_ver}
Requires:       libscotch%{so_ver}
 %endif

%description -n %{libname}
Scotch is a software package for graph and mesh/hypergraph partitioning and
sparse matrix ordering.

%package     devel
Summary:        Development libraries for scotch
Group:          Development/Libraries/C and C++
Requires:       %{libname} = %{version}
Requires:       gcc-fortran
 %if %{with mpi}
BuildRequires:  scotch-devel
Requires:       %{mpi_flavor}-devel
Requires:       scotch-devel
 %else
Provides:       scotch-devel = %{version}
 %endif

%description devel
This package contains development libraries for libscotch.

%package        devel-static
Summary:        Development libraries for scotch
Group:          Development/Libraries/C and C++
Requires:       %{package_name}-devel = %{version}
Provides:       scotch-devel-static = %{version}

%description devel-static
This package contains libscotch static libraries.

%package     -n %{pname}-%{metis}%{?my_suffix}-devel
Summary:        Development libraries for scotch
Group:          Development/Libraries/C and C++
%{?!with_mpi:Conflicts:      metis-devel}
Requires:       %{libname}
Requires:       %{package_name}-devel = %{version}

%description -n %{pname}-%{metis}%{?my_suffix}-devel
This package contains the devel libraries and header file in the case
scotch is used as a replacement of the metis library.

%prep
%setup -q -n scotch-v%{version}
cp %SOURCE1 src/Makefile.inc

%build
export CC=gcc
%{?with_mpi:source %{my_bindir}/mpivars.sh}
%{?with_mpi:export CC=mpicc}

export SUSE_ASNEEDED=0

%define CCP mpicc
%define CCD mpicc
%define CFLAGS %{optflags} -fPIC -O3 -Drestrict=__restrict -DCOMMON_FILE_COMPRESS_GZ -DCOMMON_PTHREAD -DCOMMON_RANDOM_FIXED_SEED -DSCOTCH_DETERMINISTIC -DSCOTCH_RENAME
%define LDFLAGS -pie -pthread -lz -lbz2 -lm -lrt
cd src/
make %{?_smp_mflags} %{pname} %{?with_mumps:%{?pt_pref}esmumps} CFLAGS="%CFLAGS" LDFLAGS="%LDFLAGS" CC=$CC CCP=%CCP CCD=%CCD
$CC %{LDFLAGS} -shared -Wl,-soname=lib%{pname}err.so.0 -o ../lib/lib%{pname}err.so.0.0	\
	libscotch/library_error.o
$CC %{LDFLAGS} -shared -Wl,-soname=lib%{pname}errexit.so.0 -o	\
	../lib/lib%{pname}errexit.so.0.0	libscotch/library_error_exit.o
rm -f libscotch/library_error*.o
$CC %{LDFLAGS} -shared -Wl,-soname=lib%{pname}.so.0 -o ../lib/lib%{pname}.so.0.0	\
	libscotch/*.o ../lib/lib%{pname}err.so.0.0 -lpthread -lgfortran -lz -lrt
$CC %{LDFLAGS} -shared -Wl,-soname=lib%{pname}%{metis}.so.0 -o ../lib/lib%{pname}%{metis}.so.0.0\
	libscotchmetis/*.o ../lib/lib%{pname}.so.0.0 ../lib/lib%{pname}err.so.0.0 -lz -lm -lrt
%if %{with mumps}
$CC %{LDFLAGS} -shared -Wl,-soname=lib%{?pt_pref}esmumps.so.0 -o ../lib/lib%{?pt_pref}esmumps.so.0.0	\
	esmumps/*.o
%endif
cd ..

%install
export CC=gcc
%{?with_mpi:source %{my_bindir}/mpivars.sh}
%{?with_mpi:export CC=mpicc}

pushd src/
make install prefix=%{buildroot}%{my_prefix} libdir=%{buildroot}%{my_libdir}
popd
for static_libs in lib/lib%{pname}*.a %{?with_mumps:lib/lib%{?pt_pref}esmumps.a}; do
	libs=`basename $static_libs .a`
	ln -s $libs.so.0.0 lib/$libs.so.0
	ln -s $libs.so.0.0 lib/$libs.so
done
pushd %{buildroot}%{my_libdir}
ln -s lib%{?pt_pref}scotch%{metis}.a lib%{metis}.a
ln -s lib%{?pt_pref}scotch%{metis}.so lib%{metis}.so
%if %{with mpi}
	# We create link in order to have the serial libs available in the
	# same directory as the parallel libs. A lot of software using scotch
        # can't manage different dirs for serial and parallel files during
	# their build process.
	for libs in libscotch libscotcherr libscotcherrexit libscotchmetis %{?with_mumps:libesmumps} ; do
		ln -sf %{_libdir}/$libs.so $libs.so
		ln -sf %{_libdir}/$libs.so.%{so_ver} $libs.so.%{so_ver}
		ln -sf %{_libdir}/$libs.so.%{so_ver}.0 $libs.so.%{so_ver}.0
	done
%endif
popd
%if %{without mpi}
# Add "scotch_" prefix to binaries and man pages to avoid name conficts
pushd %{buildroot}%{my_bindir}
for prog in *; do
    mv $prog scotch_${prog}
    chmod 755 scotch_$prog
done
popd
pushd %{buildroot}%{my_mandir}/man1/
for man in *; do
    mv ${man} scotch_${man}
done
popd
%endif # without mpi
cp -dp lib/lib*scotch*.so* %{?with_mumps:lib/lib*esmumps.*} %{buildroot}%{my_libdir}/
%{?with_mumps:cp include/esmumps.h %{buildroot}%{my_incdir}/}
cp src/libscotchmetis/%{metis}.h %{buildroot}%{my_incdir}/

# Convert the license files to utf8
pushd doc
iconv -f iso8859-1 -t utf-8 < CeCILL-C_V1-en.txt > CeCILL-C_V1-en.txt.conv
iconv -f iso8859-1 -t utf-8 < CeCILL-C_V1-fr.txt > CeCILL-C_V1-fr.txt.conv
mv -f CeCILL-C_V1-en.txt.conv CeCILL-C_V1-en.txt
mv -f CeCILL-C_V1-fr.txt.conv CeCILL-C_V1-fr.txt
popd

%fdupes %{buildroot}%{my_bindir}
%fdupes %{buildroot}%{my_mandir}

%if %{without mpi}
%post -n %{libname} -p /sbin/ldconfig
%postun -n %{libname} -p /sbin/ldconfig
%else

# MPI package install to non-standard locations: don't update cache
%post -n %{libname}
/sbin/ldconfig -N %{my_libdir}

%postun -n %{libname}
/sbin/ldconfig -N %{my_libdir}
%endif

%files
%doc README.txt doc/scotch*.pdf
%license doc/CeCILL-*
%{my_bindir}/%{!?with_mpi:*}
%{my_mandir}/man1/%{!?with_mpi:*}

%files -n %{libname}
%{my_libdir}/libscotch.so.*
%{my_libdir}/libscotcherr.so.*
%{my_libdir}/libscotcherrexit.so.*
%{my_libdir}/libscotchmetis.so.*
%if %{with mumps}
%{my_libdir}/libesmumps.so.*
%endif
%if %{with mpi}
%{my_libdir}/libptscotch.so.*
%{my_libdir}/libptscotcherr.so.*
%{my_libdir}/libptscotcherrexit.so.*
%{my_libdir}/libptscotchparmetis.so.*
%{?with_mumps:%{my_libdir}/libptesmumps.so.*}
%endif

%files devel
%{my_libdir}/libscotch.so
%{my_libdir}/libscotcherr.so
%{my_libdir}/libscotcherrexit.so
%{my_libdir}/libscotchmetis.so
%if %{with mumps}
%{my_libdir}/libesmumps.so
%endif
%if %{with mpi}
%{my_libdir}/libptscotch.so
%{my_libdir}/libptscotcherr.so
%{my_libdir}/libptscotcherrexit.so
%{my_libdir}/libptscotchparmetis.so
%{?with_mumps:%{my_libdir}/libptesmumps.so}
%endif
%{my_incdir}/*.h
%exclude %{my_incdir}/%{metis}.h

%files -n %{pname}-%{metis}%{?my_suffix}-devel
%{my_libdir}/lib%{metis}.so
%{my_incdir}/%{metis}.h

%files devel-static
%{my_libdir}/libscotch.a
%{my_libdir}/libscotcherr.a
%{my_libdir}/libscotcherrexit.a
%{my_libdir}/libscotchmetis.a
%if %{with mpi}
%{my_libdir}/libptscotch.a
%{my_libdir}/libptscotcherr.a
%{my_libdir}/libptscotcherrexit.a
%{my_libdir}/libptscotchparmetis.a
%{my_libdir}/libparmetis.a
%else
%{my_libdir}/libmetis.a
%endif
%if %{with mumps}
%{my_libdir}/lib%{?pt_pref}esmumps.a
%endif

%changelog
* Mon Mar 24 2025 Nicolas Morey <nicolas.morey@suse.com>
- Disable and remove support for gnu-hpc build flavours (bsc#1239982)
- Add openmpi5 build flavour
- Fix %%{metis} in package name.
* Mon Oct 23 2023 Egbert Eich <eich@suse.com>
- Disable HPC flavors on i586
* Mon Oct 23 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for obsolete openmpi[123]
- Prepare support for openmpi5
* Fri Jul  2 2021 Egbert Eich <eich@suse.com>
- Fix serial HPC build.
- Fix: 'warning: File listed twice: /usr/lib64/libscotchmetis.so*'
* Wed Jan 27 2021 Egbert Eich <eich@suse.com>
- Sync SLE and openSUSE builds (jsc#SLE-12162).
* Thu Jan 21 2021 Ana Guerrero Lopez <aguerrero@suse.com>
- Update to version 6.1.0. The change in release number reflects a change
  in behavior of the software, regarding two features:
  * Algorithms for sparse matrix reordering now fully take into account vertex
    weights. While the previous versions only accounted for them in the nested
    dissection method to compute and  optimize separator size/weight, they are
    now also accounted for in the minimum  degree and minimum fill algorithms.
    Consequently,  vertex  amalgamation  can  be  tuned so  that  the  size of
    (column-)blocks is tailored according to the real amount of computation to
    be performed on supervariables.
    The esmumps  interface  with MUMPS  has been  updated  so as to  take into
    account this new feature.
  * The routines of the vertex  graph partitioning (with overlap)  module have
  been redesigned, leading to much smaller  run time and, most often, higher
  quality
- Scotch development is moving to gitlab
- Add support for gcc10 and build support for openmpi4.
* Fri Apr 17 2020 Egbert Eich <eich@suse.com>
- only serial scotch-metis-devel package should conflict with
  metis-devel (boo#1169791).
* Thu Jan 30 2020 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Remove bogus %%undefine suffix - suffix is a RPM builtin, and
  trying to undefine it is an error since RPM 4.15.
* Tue Dec  3 2019 Egbert Eich <eich@suse.com>
- Enabled non-HPC openmpi3 build.
* Fri Nov 29 2019 Egbert Eich <eich@suse.com>
- Add support for gcc7 builds for HPC.
- Add support for gcc8 and gcc9 for HPC (jsc#SLE-7766 &
  jsc#SLE-8604).
* Thu Nov 28 2019 Egbert Eich <eich@suse.com>
- Update to version 6.0.9:
  * Scotch 6.0.7 is mostly a bugfix version. It provides some new minor
    features, such as an extension of the target architecture API and
    MeTiS v5 compatibility.
- Disable openmpi1 builds for SLE/Leap > 15.1.
- Enable openmpi3 builds for Leap and SLE > 15.1 (jsc#SLE-7773).
* Wed Nov 27 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Reflect renaming of openmpi packages to openmpi1 in Tumbleweed,
  fixes unresolvable packages.
* Tue Sep 10 2019 Ana Guerrero Lopez <aguerrero@suse.com>
- Use -ffat-lto-objects to provide a proper static library.
* Wed Aug 28 2019 Ana Guerrero Lopez <aguerrero@suse.com>
- Update to version 6.0.7: (jsc#SLE-8499)
  * bugfixes and some new minor features, such as an extension of
    the target architecture API and MeTiS v5 compatibility.
* Thu Feb 14 2019 Jan Engelhardt <jengelh@inai.de>
- Fix the package group fix.
* Mon Feb 11 2019 Egbert Eich <eich@suse.com>
- Set default module version correctly when installing
  master package, unset when deinstalling the default library
  package.
- Fix dependencies for HPC packages.
- Fix HPC modulefile: library dir is always there when module file
  is installed.
- Fix package groups.
- Cosmetic fixes to module file.
- Consolidate use of openmpi1, openmpi2, openmpi3 for non-HPC builds.
* Fri Jan 25 2019 eich@suse.com
- Disable openmpi1 and openmpi3 builds on SLE.
* Fri Jan 25 2019 eich@suse.com
- Add mpich support to HPC build.
- Reenable ldconfig for HPC & MPI:
  do not attempt to update the cache.
* Sun Jan 20 2019 eich@suse.com
- Remove 'lib' from devel packages.
- Fix dependencies.
* Mon Jan 14 2019 eich@suse.com
- Convert 'scotch' to multibuild and add HPC support.
* Mon Nov 19 2018 Mark.Olesen@esi-group.com
- Update to version 6.0.6
  * bugfix release
- Drop old patches (changed upstream).
- Use deterministic behaviour, for repeatable results
* Sun Apr  1 2018 stefan.bruens@rwth-aachen.de
- Setup MPI environment before building.
* Thu May 28 2015 foss@grueninger.de
- Update to version 6.0.4
  * bugfix release
* Sat Jan 24 2015 dmitry_r@opensuse.org
- Update to version 6.0.3
  * bugfix release
- Add "scotch_" prefix to binaries and man pages to avoid name conficts
* Thu Nov  6 2014 dmitry_r@opensuse.org
- Fix license name to SPDX format
* Sun Apr  7 2013 scorot@free.fr
- update to version 6.0.0
  * sequential graph repartitioning
  * sequential graph partitioning with fixed vertices
  * sequential graph repartitioning with fixed vertices
  * new, fast, direct k-way partitioning and mapping algorithms
  * multi-threaded, shared memory algorithms in the (formerly)
    sequential part of the library
  * exposure in the API of many distributed graph handling routines
  * embedded pseudo-random generator for improved reproducibility
- put metis and parmetis compatibilty libs and header in a
  separate package in order to avoid conflict with metis packages
- fix %%%%doc in order to not mix anymore ptscotch and scotch
  documentation files
* Thu Oct 11 2012 scorot@free.fr
- fix files list
- add metis and parmetis header files in devel devel packages
* Tue Aug 21 2012 scorot@free.fr
- fix provides of devel packages
* Sun May 13 2012 scorot@free.fr
- revert soversion change to be the same as fedora
* Tue May  8 2012 scorot@free.fr
- fix conditionnal macro error in file list
- add build of shared esmumps and ptesmumps libs
* Tue May  8 2012 scorot@free.fr
- use only the source tarball of the esmumps version
- soname version of libraries set to major release number
- put libraries in separte packages
- build ptscotch for openmpi and mvapich2
- mpi version of the libs are placed in separate packages
* Fri May  4 2012 scorot@free.fr
- spec file cleanup
- version 5.1.12
  * load balance enforcement has been improved for parallel graph
    partitioning
  * The clustering features of Scotch and PT-Scotch are now
    available through the '-q' option of command line programs
    dgmap, dgpart, gmap and gpart.
  * Strategy string building helper routines are also available
    for clustering in the libScotch library.
  * Some internal graph handling routines have been exposed in
    the interface, such as for graph coarsening.
* Sun Nov  8 2009 andrea@opensuse.org
- New package
